module MenuDevice(menuFunctions) where


--	********************************************************************************
--	Clean to Haskell Standard Object I/O library, version 1.2
--	********************************************************************************


import	OSMenu
import	OSTypes(osNoWindowPtr)
import	OSEvent(MsgEvent(..))
import	CommonDef
import	StdId
import	StdGUI
import	IOState
import	MenuCreate
import	MenuEvent
import	MenuHandle
import	MenuDefAccess
import	MenuAccess(menuStateHandleGetMenuId)
import	StdMenuAttribute(isMenuFunction, getMenuFun, isMenuModsFunction, getMenuModsFun)
import  ReceiverHandle
import  ReceiverTable(RecLoc(..))
import	StdProcessAttribute(getProcessToolbarAtt, isProcessToolbar)
import	Monad(when)
import  IOExts(fixIO)


menuDeviceFatalError :: String -> String -> x
menuDeviceFatalError rule error = dumpFatalError rule "MenuDevice" error

menuFunctions :: DeviceFunctions ps
menuFunctions = DeviceFunctions
	  {	dDevice	= MenuDevice
	  ,	dShow	= menuShow
	  ,	dHide	= menuHide
	  ,	dEvent	= menuEvent
	  ,	dDoIO	= menuIO
	  ,	dOpen	= menuOpen
	  ,	dClose	= menuClose
	  }

menuShow :: ps -> GUI ps ps
menuShow ps = do	
	osdinfo <- accIOEnv ioStGetOSDInfo
	(let 
		maybeMenuBar = getOSDInfoOSMenuBar osdinfo
		osMenuBar				= fromJust maybeMenuBar
	 in 
		if isNothing maybeMenuBar
	 	then menuDeviceFatalError "menuFunctions.dShow" "OSMenuBar could not be retrieved from OSDInfo"
	 	else do	
			liftIO (osMenuBarSet osMenuBar)		  	
		  	appIOEnv (ioStSetOSDInfo (setOSDInfoOSMenuBar osMenuBar osdinfo)))
	return ps

menuClose :: ps -> GUI ps ps
menuClose ps = do
	osdinfo <- accIOEnv ioStGetOSDInfo
	(case getOSDInfoOSMenuBar osdinfo of
		Nothing -> menuDeviceFatalError "MenuFunctions.dClose" "OSMenuBar could not be retrieved from OSDInfo"
		Just osMenuBar -> do
			(found,menus) <- accIOEnv (ioStGetDevice MenuDevice)
			when (not found) (menuDeviceFatalError "MenuFunctions.dClose" "could not retrieve MenuSystemState from IOSt")				
			let mHs	= menuSystemStateGetMenuHandles menus
			liftIO (disposeMenuHandles mHs osMenuBar)
			ioid <- accIOEnv ioStGetIOId
			rt <- ioStGetReceiverTable
			it <- ioStGetIdTable
			let (rt1,it1) = foldr (disposeIds ioid) (rt,it) (mMenus mHs)
			ioStSetIdTable it1
			ioStSetReceiverTable rt1
			appIOEnv (ioStRemoveDevice MenuDevice)
			appIOEnv (ioStRemoveDeviceFunctions MenuDevice)
			return ps)
	where
		disposeIds :: SystemId -> MenuStateHandle ps -> (ReceiverTable,IdTable) -> (ReceiverTable,IdTable)
		disposeIds ioid (MenuStateHandle (MenuLSHandle {mlsHandle=mH})) ts =
			foldr (disposeMenuIds ioid) ts (mItems mH)
			
menuHide :: ps -> GUI ps ps
menuHide ps = do	
	(found,menus) <- accIOEnv (ioStGetDevice MenuDevice)	
	when found (liftIO (osMenuBarClear) >> appIOEnv (ioStSetDevice menus))
	return ps


{-	Opening menus:
	Note:	all interactive processes have atleast the AppleMenu to identify the 
			process, and is used for checking whether the process is active
			(see IOStIsActive further down this module).
			If the process is a subprocess, then the ioguishare of its IOSt
			contains the list to the Mac toolbox menu system.
-}
menuOpen :: ps -> GUI ps ps
menuOpen ps = do
	hasMenu <- accIOEnv (ioStHasDevice MenuDevice)
	(if hasMenu then return ps
	 else do
		di <- accIOEnv ioStGetDocumentInterface
		popUpId <- getPopUpId di
		let bound = case di of
			NDI -> Finite 0
			SDI -> Infinite
			MDI -> Infinite
		let mHs	= MenuHandles
			{ mMenus	= []
			, mKeys		= []			
			, mEnabled	= systemAble
			, mNrMenuBound	= bound
			, mPopUpId	= popUpId					
			}
		appIOEnv (ioStSetDevice (MenuSystemState mHs))
		appIOEnv (ioStSetDeviceFunctions menuFunctions)
		return ps)
	where
		getPopUpId :: DocumentInterface -> GUI ps (Maybe Id)
		getPopUpId NDI = return Nothing
		getPopUpId _ = fmap Just openId


menuIO :: DeviceEvent -> ps -> GUI ps ps
menuIO deviceEvent ps = do
	ok <- accIOEnv (ioStHasDevice MenuDevice)
	when (not ok) (menuDeviceFatalError "menuFunctions.dDoIO" "could not retrieve MenuSystemState from IOSt")
	toGUI (menuIO deviceEvent ps)
	where
		menuIO :: DeviceEvent -> ps -> IOSt ps -> IO (ps,IOSt ps)
		menuIO (ReceiverEvent msgEvent) ps ioState =
			let
				(_,mDevice) = ioStGetDevice MenuDevice ioState
				menus   = menuSystemStateGetMenuHandles mDevice
			in
				menuMsgIO msgEvent menus ps ioState
			where		
				menuMsgIO :: MsgEvent -> MenuHandles ps -> ps -> IOSt ps -> IO (ps, IOSt ps)
				menuMsgIO msgEvent menus@(MenuHandles {mMenus=mHs}) ps ioState = do
					menusMsgIO (\mHs ioState -> (ioStSetDevice (MenuSystemState menus{mMenus=mHs})) ioState) (rlParentId (getMsgEventRecLoc msgEvent)) msgEvent mHs ps ioState					
					where
						menusMsgIO :: ([MenuStateHandle ps] -> IOSt ps -> IOSt ps) ->
							      Id -> MsgEvent -> [MenuStateHandle ps] -> ps -> IOSt ps -> IO (ps, IOSt ps)
						menusMsgIO build menuId msgEvent [] ps ioState =
							menuDeviceFatalError "menuIO (ReceiverEvent _) _" "menu could not be found"
						menusMsgIO build menuId msgEvent (msH:msHs) ps ioState =
							if menuStateHandleGetMenuId msH == menuId
							then menuStateMsgIO (\msH ioState -> build (msH:msHs) ioState) msgEvent msH ps ioState								
							else menusMsgIO (\msHs ioState -> build (msH:msHs) ioState) menuId msgEvent msHs ps ioState
								

		menuIO (MenuTraceEvent info) ps ioState =
			let
				(_,mDevice) = ioStGetDevice MenuDevice ioState
				menus = menuSystemStateGetMenuHandles mDevice
			in
				menuTraceIO info menus ps ioState
			where
				menuTraceIO :: MenuTraceInfo -> MenuHandles ps -> ps -> IOSt ps -> IO (ps, IOSt ps)
				menuTraceIO info@(MenuTraceInfo {mtId=mtId}) menus@(MenuHandles {mMenus=mHs}) ps ioState = do
					menusTraceIO (\mHs ioState -> (ioStSetDevice (MenuSystemState menus{mMenus=mHs})) ioState) mtId info mHs ps ioState
					where
						menusTraceIO :: ([MenuStateHandle ps] -> IOSt ps -> IOSt ps) ->
								Id -> MenuTraceInfo -> [MenuStateHandle ps] -> ps -> IOSt ps -> IO (ps, IOSt ps)
						menusTraceIO build menuId info [] ps ioState =
							menuDeviceFatalError "menuIO (MenuTraceEvent _) _" "menu could not be found"
						menusTraceIO build menuId info (msH:msHs) ps ioState =
							if menuStateHandleGetMenuId msH == menuId
							then menuStateTraceIO (\msH ioState -> build (msH:msHs) ioState) info msH ps ioState
							else menusTraceIO (\msHs ioState -> build (msH:msHs) ioState) menuId info msHs ps ioState
								

		menuIO (ToolbarSelection tbsItemNr) ps ioState =
			let 
				atts     	    = ioStGetProcessAttributes ioState
				(hasToolbarAtt,att) = cselect isProcessToolbar undefined atts
				toolbarItems	    = getProcessToolbarAtt att
			in
				if not hasToolbarAtt
				then return (ps,ioState)
				else fromGUI (getToolbarFunction tbsItemNr toolbarItems ps) ioState					
			where
				getToolbarFunction :: Int -> [ToolbarItem ps] -> ps -> GUI ps ps
				getToolbarFunction _ [] =
					menuDeviceFatalError "menuIO (ToolbarSelection _)" "toolbar index out of range"
				getToolbarFunction i (ToolbarSeparator:items) = getToolbarFunction i items
				getToolbarFunction i ((ToolbarItem _ _ f):items)
					| i==1 	= f
					| otherwise		= getToolbarFunction (i-1) items
		menuIO _ _ _
			= menuDeviceFatalError "menuIO" "unexpected DeviceEvent"



{-	Apply the Menu(Mods)Function of a selected menu item. -}

menuStateTraceIO :: (MenuStateHandle ps -> IOSt ps -> IOSt ps) -> 
		    MenuTraceInfo -> MenuStateHandle ps -> ps -> IOSt ps -> IO (ps, IOSt ps)
menuStateTraceIO build info@(MenuTraceInfo {mtParents=mtParents}) (MenuStateHandle (MenuLSHandle {mlsState=ls,mlsHandle=mH})) ps ioState = do
	r <- subMenusTraceIO (\itemHs st ioState -> build (MenuStateHandle (MenuLSHandle {mlsState=fst st,mlsHandle=mH{mItems=itemHs}})) ioState) (mHandle mH) mtParents (mItems mH) (ls,ps) ioState
	let ((_,ps1),ioState) = r
	return (ps1,ioState)
	where
		--	subMenusTraceIO finds the final submenu that contains the selected menu item and then applies its Menu(Mods)Function.
		subMenusTraceIO :: ([MenuElementHandle ls ps] -> (ls,ps) -> IOSt ps -> IOSt ps) -> 
				   OSMenu -> [Int] -> [MenuElementHandle ls ps] -> (ls,ps) -> IOSt ps -> IO ((ls,ps), IOSt ps)
		subMenusTraceIO build mHandle [] itemHs ls_ps ioState = do
			r <- menuElementsTraceIO build (mtItemNr info) mHandle 0 itemHs ls_ps ioState
			let (_,ls_ps1,ioState) = r
			return (ls_ps1,ioState)
		subMenusTraceIO build mHandle (subIndex:subIndices) itemHs ls_ps ioState = do
			r <- subMenuTraceIO build subIndex mHandle subIndices 0 itemHs ls_ps ioState
			let (_,ls_ps1,ioState) = r
			return (ls_ps1,ioState)
			where
				subMenuTraceIO :: ([MenuElementHandle ls ps] -> (ls,ps) -> IOSt ps -> IOSt ps) ->
						  Int -> OSMenu -> [Int] -> Int -> [MenuElementHandle ls ps] -> (ls,ps) -> IOSt ps -> IO (Int,(ls,ps),IOSt ps)
				subMenuTraceIO build parentIndex mHandle parentsIndex zIndex (itemH:itemHs) ls_ps ioState = do
					r <- subMenuTraceIO' (\itemH st ioState -> build (itemH:itemHs) st ioState) parentIndex mHandle parentsIndex zIndex itemH ls_ps ioState
					let (zIndex1,ls_ps1,ioState) = r
					(if parentIndex<zIndex1 then return r
					 else subMenuTraceIO (\itemHs st ioState -> build (itemH:itemHs) st ioState) parentIndex mHandle parentsIndex zIndex1 itemHs ls_ps1 ioState)
					where
						subMenuTraceIO' :: (MenuElementHandle ls ps -> (ls,ps) -> IOSt ps -> IOSt ps) ->
								   Int -> OSMenu -> [Int] -> Int -> MenuElementHandle ls ps -> (ls,ps) -> IOSt ps -> IO (Int,(ls,ps),IOSt ps)
						subMenuTraceIO' build parentIndex mHandle parentsIndex zIndex subH@(SubMenuHandle {mSubHandle=mSubHandle, mSubItems=itemHs}) ls_ps ioState
							| parentIndex /= zIndex = return (zIndex+1,ls_ps,ioState)
							| otherwise = do
								r <- subMenusTraceIO (\itemHs st ioState -> build subH{mSubItems=itemHs} st ioState) mSubHandle parentsIndex itemHs ls_ps ioState
								let (ls_ps1,ioState) = r
								return (zIndex+1,ls_ps1,ioState)
						subMenuTraceIO' build parentIndex mHandle parentsIndex zIndex radioH@(RadioMenuHandle {mRadioItems=itemHs}) ls_ps ioState =
							return (zIndex+length itemHs,ls_ps,ioState)
						subMenuTraceIO' build parentIndex mHandle parentsIndex zIndex itemH@(MenuItemHandle {}) ls_ps ioState =
							return (zIndex+1,ls_ps,ioState)
						subMenuTraceIO' build parentIndex mHandle parentsIndex zIndex (MenuListLSHandle itemHs) ls_ps ioState =
							subMenuTraceIO (\itemHs st ioState -> build (MenuListLSHandle itemHs) st ioState) parentIndex mHandle parentsIndex zIndex itemHs ls_ps ioState
						subMenuTraceIO' build parentIndex mHandle parentsIndex zIndex (MenuExtendLSHandle exLS itemHs) (ls,ps) ioState = do
							r <- subMenuTraceIO (\itemHs st ioState -> build (MenuExtendLSHandle (fst (fst st)) itemHs) (snd (fst st),snd st) ioState) parentIndex mHandle parentsIndex zIndex itemHs ((exLS,ls),ps) ioState
							let (zIndex1,((_,ls1),ps1),ioState) = r
							return (zIndex,(ls1,ps1),ioState)
						subMenuTraceIO' build parentIndex mHandle parentsIndex zIndex (MenuChangeLSHandle chLS itemHs) (ls,ps) ioState = do
							r <- subMenuTraceIO (\itemHs st ioState -> build (MenuChangeLSHandle (fst st) itemHs) (ls,snd st) ioState) parentIndex mHandle parentsIndex zIndex itemHs (chLS,ps) ioState
							let (zIndex1,(_,ps1),ioState) = r
							return (zIndex,(ls,ps1),ioState)
						subMenuTraceIO' _ _ _ _ zIndex itemH ls_ps ioState =
							return (zIndex,ls_ps,ioState)
				subMenuTraceIO _ _ _ _ zIndex itemHs ls_ps ioState =
					return (zIndex,ls_ps,ioState)
		
		--	menuElementsTraceIO applies the Menu(Mods)Function of the menu item at index itemIndex to the context state.
		menuElementsTraceIO :: ([MenuElementHandle ls ps] -> (ls,ps) -> IOSt ps -> IOSt ps) ->
				       Int -> OSMenu -> Int -> [MenuElementHandle ls ps] -> (ls,ps) -> IOSt ps -> IO (Int,(ls,ps),IOSt ps)
		menuElementsTraceIO build itemIndex mHandle zIndex (itemH:itemHs) ls_ps ioState = do
			r <- menuElementTraceIO (\itemH st ioState -> build (itemH:itemHs) st ioState) itemIndex mHandle zIndex itemH ls_ps ioState
			let (zIndex1,ls_ps1,ioState) = r
			(if itemIndex<zIndex1 then return r
			 else menuElementsTraceIO (\itemHs st ioState -> build (itemH:itemHs) st ioState) itemIndex mHandle zIndex1 itemHs ls_ps1 ioState)
			where
				menuElementTraceIO :: (MenuElementHandle ls ps -> (ls,ps) -> IOSt ps -> IOSt ps) ->
						      Int -> OSMenu -> Int -> MenuElementHandle ls ps -> (ls,ps) -> IOSt ps -> IO (Int, (ls,ps), IOSt ps)
				menuElementTraceIO build itemIndex mHandle zIndex itemH@(MenuItemHandle {mItemAtts=mItemAtts}) (ls,ps) ioState
					| itemIndex /= zIndex || not hasFun	= 
						return (zIndex+1,(ls,ps),ioState)
					| otherwise = do
						r <- fixIO (\st -> fromGUI (f (ls,ps)) (build itemH (fst st) ioState))
						let ((ls1,ps1), ioState) = r
						return (zIndex+1,(ls1,ps1),ioState)
					where
						(hasFun,fAtt) = cselect isEitherFun undefined mItemAtts
						f	= if isMenuFunction fAtt
							  then getMenuFun fAtt
							  else getMenuModsFun fAtt (mtModifiers info)
						isEitherFun f = isMenuFunction f || isMenuModsFunction f
				menuElementTraceIO build itemIndex mHandle zIndex radioH@(RadioMenuHandle {mRadioItems=itemHs}) ls_ps ioState =
					let nrRadios = length itemHs
					in
						if itemIndex>=zIndex+nrRadios then
							-- Selected item is not one of these radio items
							return (zIndex+nrRadios,ls_ps,ioState)
						else
						    let 
							curIndex	= mRadioIndex radioH
							newIndex	= mtItemNr info - zIndex + 1
							curH 		= mOSMenuItem (itemHs!!(curIndex-1))
							newH 		= mOSMenuItem (itemHs!!(newIndex-1))
						    in do
							  osMenuItemCheck False mHandle curH
							  osMenuItemCheck True  mHandle newH
							  r <- menuElementsTraceIO (\itemHs st ioState -> build radioH{mRadioItems=itemHs, mRadioIndex=newIndex} st ioState) itemIndex mHandle zIndex itemHs ls_ps ioState
							  let (_,ls_ps,ioState) = r
							  return (zIndex+length itemHs,ls_ps,ioState)
								
				menuElementTraceIO build itemIndex mHandle zIndex itemH@(SubMenuHandle {}) ls_ps ioState =
					return (zIndex+1,ls_ps,ioState)
					
				menuElementTraceIO build itemIndex mHandle zIndex (MenuListLSHandle itemHs) ls_ps ioState =
					menuElementsTraceIO (\itemHs st ioState -> build (MenuListLSHandle itemHs) st ioState) itemIndex mHandle zIndex itemHs ls_ps ioState
					
				menuElementTraceIO build itemIndex mHandle zIndex (MenuExtendLSHandle exLS itemHs) (ls,ps) ioState = do
					r <- menuElementsTraceIO (\itemHs st ioState -> build (MenuExtendLSHandle (fst (fst st)) itemHs) (snd (fst st),snd st) ioState) itemIndex mHandle zIndex itemHs ((exLS,ls),ps) ioState
					let (zIndex,((_,ls),ps),ioState) = r
					return (zIndex,(ls,ps),ioState)
				menuElementTraceIO build itemIndex mHandle zIndex (MenuChangeLSHandle chLS itemHs) (ls,ps) ioState = do
					r <- menuElementsTraceIO (\itemHs st ioState -> build (MenuChangeLSHandle (fst st) itemHs) (ls,snd st) ioState) itemIndex mHandle zIndex itemHs (chLS,ps) ioState
					let (zIndex,(_,ps),ioState) = r
					return (zIndex,(ls,ps),ioState)
				menuElementTraceIO _ _ _ zIndex itemH ls_ps ioState =
					return (zIndex,ls_ps,ioState)
		menuElementsTraceIO _ _ _ zIndex [] ls_ps ioState = 
			return (zIndex,ls_ps,ioState)


{-	menuStateMsgIO handles all message events. -}

menuStateMsgIO :: (MenuStateHandle ps -> IOSt ps -> IOSt ps) ->
		  MsgEvent -> MenuStateHandle ps -> ps -> IOSt ps -> IO (ps,IOSt ps)
menuStateMsgIO build msgEvent msH@(MenuStateHandle mlsH@(MenuLSHandle {mlsState=ls,mlsHandle=mH})) ps ioState =
    let 
    	rId	= rlReceiverId (getMsgEventRecLoc msgEvent)
    	action	= case msgEvent of
--	   (QASyncMessage _) -> menuQASyncIO rId msgEvent
	   ( ASyncMessage _) -> menuASyncIO  (\mH st ioState -> build (MenuStateHandle mlsH{mlsState=fst st,mlsHandle=mH}) ioState) rId msgEvent
--	   (  SyncMessage _) -> menuSyncIO   rId msgEvent
    in
    	do
    		r <- action mH (ls,ps) ioState
    		let ((_,ps1),ioState) = r
    		return (ps1,ioState)
    where	

{-	//	menuQASyncIO queues an asynchronous message in the message queue of the indicated receiver menu element.
	menuQASyncIO :: !Id !QASyncMessage !(MenuHandle .ls .pst) (.ls,.pst) -> (!MsgEvent,!MenuHandle .ls .pst,(.ls,.pst))
	menuQASyncIO rId msg mH=:{mItems} ls_ps
		= (QASyncMessage msg,{mH & mItems=itemHs},ls_ps)
	where
		(_,itemHs)	= elementsQASyncIO rId msg.qasmMsg mItems
		
		elementsQASyncIO :: !Id !SemiDynamic ![MenuElementHandle .ls .pst] -> (!Bool,![MenuElementHandle .ls .pst])
		elementsQASyncIO rId msg [itemH:itemHs]
			# (done,itemH)		= elementQASyncIO rId msg itemH
			| done
				= (done,[itemH:itemHs])
			| otherwise
				# (done,itemHs)	= elementsQASyncIO rId msg itemHs
				= (done,[itemH:itemHs])
		where
			elementQASyncIO :: !Id !SemiDynamic !(MenuElementHandle .ls .pst) -> (!Bool,!MenuElementHandle .ls .pst)
			elementQASyncIO rId msg mrH=:(MenuReceiverHandle itemH=:{mReceiverHandle=rH})
				| rId<>rH.rId	= (False,mrH)
				| otherwise		= (True,MenuReceiverHandle {itemH & mReceiverHandle=receiverAddASyncMessage rId msg rH})
			elementQASyncIO rId msg (SubMenuHandle itemH=:{mSubItems=itemHs})
				# (done,itemHs)	= elementsQASyncIO rId msg itemHs
				= (done,SubMenuHandle {itemH & mSubItems=itemHs})
			elementQASyncIO rId msg (MenuListLSHandle itemHs)
				# (done,itemHs)	= elementsQASyncIO rId msg itemHs
				= (done,MenuListLSHandle itemHs)
			elementQASyncIO rId msg (MenuExtendLSHandle	mExH=:{mExtendItems=itemHs})
				# (done,itemHs)	= elementsQASyncIO rId msg itemHs
				= (done,MenuExtendLSHandle {mExH & mExtendItems=itemHs})
			elementQASyncIO rId msg (MenuChangeLSHandle	mChH=:{mChangeItems=itemHs})
				# (done,itemHs)	= elementsQASyncIO rId msg itemHs
				= (done,MenuChangeLSHandle {mChH & mChangeItems=itemHs})
			elementQASyncIO rId msg itemH
				= (False,itemH)
		elementsQASyncIO _ _ _
			= (False,[])
-}
	-- menuASyncIO handles the first asynchronous message in the message queue of the indicated receiver menu element.
	menuASyncIO :: (MenuHandle ls ps -> (ls,ps) -> IOSt ps -> IOSt ps) ->
		       Id -> MsgEvent -> MenuHandle ls ps -> (ls,ps) -> IOSt ps -> IO ((ls,ps), IOSt ps)
	menuASyncIO build rId msg mH@(MenuHandle {mItems=itemHs}) ls_ps ioState = do
	    r <- elementsASyncIO (\itemHs st ioState -> build mH{mItems=itemHs} st ioState) rId itemHs ls_ps ioState
	    let (_,ls_ps,ioState) = r
	    return (ls_ps,ioState)
	    where
		elementsASyncIO :: ([MenuElementHandle ls ps] -> (ls,ps) -> IOSt ps -> IOSt ps) ->
				   Id -> [MenuElementHandle ls ps] -> (ls,ps) -> IOSt ps -> IO (Bool,(ls,ps),IOSt ps)
		elementsASyncIO build rId (itemH:itemHs) ls_ps ioState = do
			r <- elementASyncIO (\itemH st ioState -> build (itemH:itemHs) st ioState) rId itemH ls_ps ioState
			let (done,ls_ps,ioState) = r
			(if done then return r
			 else elementsASyncIO (\itemHs st ioState -> build (itemH:itemHs) st ioState) rId itemHs ls_ps ioState)
			where
				elementASyncIO :: (MenuElementHandle ls ps -> (ls,ps) -> IOSt ps -> IOSt ps) ->
						  Id -> MenuElementHandle ls ps -> (ls,ps) -> IOSt ps -> IO (Bool,(ls,ps),IOSt ps)
				elementASyncIO build id mrH@(MenuReceiverHandle rH@(ReceiverHandle {rId=rId}) atts) ls_ps ioState
					| id /= rId = return (False,ls_ps,ioState)
					| otherwise = do
						r <- fixIO (\st -> fromGUI (receiverASyncIO rH ls_ps) (build mrH (fst st) ioState))
						let (ls_ps, ioState) = r
						return (True,ls_ps,ioState)
					where
						receiverASyncIO :: ReceiverHandle m r ls ps -> GUIFun ls ps
						receiverASyncIO rH@(ReceiverHandle {rASMQ=channel,rFun=rFun}) ls_ps = do
							msg <- liftIO (readChan channel)
							(_,ls_ps1) <- rFun msg ls_ps
							return ls_ps1

				elementASyncIO build rId subH@(SubMenuHandle {mSubItems=itemHs}) ls_ps ioState = do
					elementsASyncIO (\itemHs st ioState -> build subH{mSubItems=itemHs} st ioState) rId itemHs ls_ps ioState
					
				elementASyncIO build rId (MenuListLSHandle itemHs) ls_ps ioState = do
					elementsASyncIO (\itemHs st ioState -> build (MenuListLSHandle itemHs) st ioState) rId itemHs ls_ps ioState

				elementASyncIO build rId (MenuExtendLSHandle exLS itemHs) (ls,ps) ioState = do
					r <- elementsASyncIO (\itemHs st ioState -> build (MenuExtendLSHandle (fst (fst st)) itemHs) (snd (fst st),snd st) ioState) rId itemHs ((exLS,ls),ps) ioState
					let (done,((_,ls),ps),ioState) = r
					return (done,(ls,ps),ioState)
				elementASyncIO build rId (MenuChangeLSHandle chLS itemHs) (ls,ps) ioState = do
					r <- elementsASyncIO (\itemHs st ioState -> build (MenuChangeLSHandle (fst st) itemHs) (ls,snd st) ioState) rId itemHs (chLS,ps) ioState
					let (done,(_,ps),ioState) = r
					return (done,(ls,ps),ioState)
				elementASyncIO build _ itemH ls_ps ioState =
					return (False,ls_ps,ioState)
		elementsASyncIO _ _ [] ls_ps ioState =
			return (False,ls_ps,ioState)
{-
	//	menuSyncIO lets the indicated receiver control handle the synchronous message.
	menuSyncIO :: !Id !SyncMessage !(MenuHandle .ls .pst) (.ls,.pst) -> (!MsgEvent,MenuHandle .ls .pst,(.ls,.pst))
	menuSyncIO r2Id msg mH=:{mItems} ls_ps
		= (SyncMessage msg1,{mH & mItems=itemHs},ls_ps1)
	where
		(_,msg1,itemHs,ls_ps1)	= elementsSyncIO r2Id msg mItems ls_ps
		
		elementsSyncIO :: !Id !SyncMessage ![MenuElementHandle .ls .pst] (.ls,.pst)
					-> (!Bool,!SyncMessage, [MenuElementHandle .ls .pst],(.ls,.pst))
		elementsSyncIO r2Id msg [itemH:itemHs] ls_ps
			# (done,msg,itemH,ls_ps)		= elementSyncIO r2Id msg itemH ls_ps
			| done
				= (done,msg,[itemH:itemHs],ls_ps)
			| otherwise
				# (done,msg,itemHs,ls_ps)	= elementsSyncIO r2Id msg itemHs ls_ps
				= (done,msg,[itemH:itemHs],ls_ps)
		where
			elementSyncIO :: !Id !SyncMessage !(MenuElementHandle .ls .pst) (.ls,.pst)
					   -> (!Bool,!SyncMessage,  MenuElementHandle .ls .pst, (.ls,.pst))
			elementSyncIO rId msg mrH=:(MenuReceiverHandle itemH=:{mReceiverHandle=rH}) ls_ps
				| rId<>rH.rId
					= (False,msg,mrH,ls_ps)
				| otherwise
					# (msg,rH,ls_ps)= receiverSyncIO msg rH ls_ps
					= (True,msg,MenuReceiverHandle {itemH & mReceiverHandle=rH},ls_ps)
			where
				receiverSyncIO :: !SyncMessage !(ReceiverHandle .ls .pst) (.ls,.pst)
							  -> (!SyncMessage,  ReceiverHandle .ls .pst, (.ls,.pst))
				receiverSyncIO msg rH ls_ps
					# (response,rH,ls_ps)	= receiverHandleSyncMessage msg rH ls_ps
					= ({msg & smResp=response},rH,ls_ps)
			elementSyncIO rId msg (SubMenuHandle itemH=:{mSubItems=itemHs}) ls_ps
				# (done,msg,itemHs,ls_ps)	= elementsSyncIO rId msg itemHs ls_ps
				= (done,msg,SubMenuHandle {itemH & mSubItems=itemHs},ls_ps)
			elementSyncIO rId msg (MenuListLSHandle itemHs) ls_ps
				# (done,msg,itemHs,ls_ps)	= elementsSyncIO rId msg itemHs ls_ps
				= (done,msg,MenuListLSHandle itemHs,ls_ps)
			elementSyncIO rId msg (MenuExtendLSHandle {mExtendLS=ls1,mExtendItems=itemHs}) (ls,ps)
				# (done,msg,itemHs,((ls1,ls),ps))	= elementsSyncIO rId msg itemHs ((ls1,ls),ps)
				= (done,msg,MenuExtendLSHandle {mExtendLS=ls1,mExtendItems=itemHs},(ls,ps))
			elementSyncIO rId msg (MenuChangeLSHandle {mChangeLS=ls1,mChangeItems=itemHs}) (ls,ps)
				# (done,msg,itemHs,(ls1,ps))	= elementsSyncIO rId msg itemHs (ls1,ps)
				= (done,msg,MenuChangeLSHandle {mChangeLS=ls1,mChangeItems=itemHs},(ls,ps))
			elementSyncIO rId msg itemH ls_ps
				= (False,msg,itemH,ls_ps)
		elementsSyncIO _ msg _ ls_ps
			= (False,msg,[],ls_ps)
-}