{-# OPTIONS -cpp #-}

module WindowAccess ( module WindowAccess, module WindowHandle ) where


--	********************************************************************************
--	Clean to Haskell Standard Object I/O library, version 1.2
--	
--	WindowAccess defines access operations to Window(State)Handle(s).
--	********************************************************************************


import CommonDef
import OSSystem
import WindowHandle
import List(find)
import KeyFocus
import StdControlAttribute(isControlKeyboard)
import WindowCrossCall_12
import GlaExts(unsafeCoerce#)

#include "preprocess.h"
         -- Compile with -cpp; set MVAR=1 for MVAR version


windowaccessFatalError :: String -> String -> x
windowaccessFatalError function error
	= dumpFatalError function "WindowAccess" error


--	Dummy values for window handles.

dummyWindowHandles :: WindowHandles IF_MVAR(,ps)
dummyWindowHandles
	= WindowHandles
		{ whsWindows       = undefined
		, whsNrWindowBound = Infinite
		, whsFinalModalLS  = []
		, whsModal	   = False
		}

dummyWindowStateHandle :: WindowStateHandle IF_MVAR(,ps)
dummyWindowStateHandle
	= WindowStateHandle undefined undefined

#if MVAR
#else
dummyWindowLSHandle :: WindowLSHandle ls ps
dummyWindowLSHandle
	= WindowLSHandle
		{ wlsState    = undefined
		, wlsHandle   = undefined
		}
#endif

dummyWindowHandle :: WindowHandle IF_MVAR(,ls ps)
dummyWindowHandle
	= WindowHandle
		{ whMode      = undefined
		, whKind      = undefined
		, whTitle     = undefined
		, whItemNrs   = undefined
		, whKeyFocus  = undefined
		, whWindowInfo= undefined
		, whItems     = undefined
		, whShow      = undefined
		, whAtts      = undefined
		, whSelect    = undefined
		, whDefaultId = undefined
		, whCancelId  = undefined
		, whSize      = undefined
		, whClosing   = undefined
		}

#if MVAR
initWindowHandle :: Title -> WindowMode -> WindowKind -> WindowInfo -> [WElementHandle] -> [WindowAttribute] -> WindowHandle
#else
initWindowHandle :: Title -> WindowMode -> WindowKind -> WindowInfo -> [WElementHandle ls ps] -> [WindowAttribute ls ps] -> WindowHandle ls ps
#endif
initWindowHandle title mode kind info itemHs atts
	= WindowHandle
		{ whMode      = mode
		, whKind      = kind
		, whTitle     = title
		, whItemNrs   = [1..]
		, whKeyFocus  = KeyFocus {kfItem=Nothing,kfItems=[]}
		, whWindowInfo= info
		, whItems     = itemHs
		, whShow      = True
		, whAtts      = atts
		, whSelect    = True
		, whDefaultId = Nothing
		, whCancelId  = Nothing
		, whSize      = zero
		, whClosing   = False
		}
		
--  Access to the particular WindowInfo alternatives (partial functions!).

getWindowInfoWindowData :: WindowInfo -> WindowData
getWindowInfoWindowData (WindowInfo wData) = wData

--	Access to the additional WItemInfo field of a WItemHandle (partial functions!).

getWItemRadioInfo :: WItemInfo IF_MVAR(,ls ps) -> RadioInfo IF_MVAR(,ls ps)
getWItemRadioInfo (WRadioInfo info) = info

getWItemCheckInfo :: WItemInfo IF_MVAR(,ls ps) -> CheckInfo IF_MVAR(,ls ps)
getWItemCheckInfo (WCheckInfo info) = info

getWItemPopUpInfo :: WItemInfo IF_MVAR(,ls ps) -> PopUpInfo IF_MVAR(,ls ps)
getWItemPopUpInfo (WPopUpInfo info) = info

getWItemSliderInfo :: WItemInfo IF_MVAR(,ls ps) -> SliderInfo IF_MVAR(,ls ps)
getWItemSliderInfo (WSliderInfo info) = info

getWItemTextInfo :: WItemInfo IF_MVAR(,ls ps) -> TextInfo
getWItemTextInfo (WTextInfo info) = info

getWItemEditInfo :: WItemInfo IF_MVAR(,ls ps) -> EditInfo
getWItemEditInfo (WEditInfo info) = info

getWItemButtonInfo :: WItemInfo IF_MVAR(,ls ps) -> ButtonInfo
getWItemButtonInfo (WButtonInfo info) = info

getWItemCustomButtonInfo :: WItemInfo IF_MVAR(,ls ps) -> CustomButtonInfo
getWItemCustomButtonInfo (WCustomButtonInfo info) = info

getWItemCustomInfo :: WItemInfo IF_MVAR(,ls ps) -> CustomInfo
getWItemCustomInfo (WCustomInfo info) = info

getWItemCompoundInfo :: WItemInfo IF_MVAR(,ls ps) -> CompoundInfo
getWItemCompoundInfo (WCompoundInfo info) = info



--	For internal identification of windows/dialogs Id and OSWindowPtr (Integer) can be used.

data	WID				-- Identify a window/dialog either
	= ById  !Id			-- by its Id, or
	| ByPtr !OSWindowPtr		-- by its OSWindowPtr

class ToWID x where
	toWID :: x -> WID

instance ToWID Id where
	toWID id = ById id
instance ToWID Int where
	toWID wPtr = ByPtr wPtr
instance ToWID WIDS where
	toWID wids = ByPtr (wPtr wids)

widbyId :: WID -> Bool
widbyId (ById _) = True
widbyId _        = False

widbyPtr :: WID -> Bool
widbyPtr (ByPtr _) = True
widbyPtr _         = False

widgetId :: WID -> Id
widgetId (ById id) = id

widgetPtr :: WID -> OSWindowPtr
widgetPtr (ByPtr ptr) = ptr

identifyWIDS :: WID -> WIDS -> Bool
identifyWIDS (ById  id)  (WIDS {wId=wId})   = id==wId
identifyWIDS (ByPtr ptr) (WIDS {wPtr=wPtr}) = ptr==wPtr

identifyMaybeId :: Id -> Maybe Id -> Bool
identifyMaybeId id (Just id') = id==id'
identifyMaybeId _ _ = False


--	Transforming CursorShape to OS cursor codes:
toCursorCode :: CursorShape -> Int
toCursorCode StandardCursor	= cursARROW
toCursorCode BusyCursor		= cursBUSY
toCursorCode IBeamCursor	= cursIBEAM
toCursorCode CrossCursor	= cursCROSS
toCursorCode FatCrossCursor	= cursFATCROSS
toCursorCode ArrowCursor	= cursARROW
toCursorCode HiddenCursor	= cursHIDDEN


--	Calculating the view frame of window/compound with visibility of scrollbars.

getCompoundContentRect :: OSWindowMetrics -> (Bool,Bool) -> Rect -> Rect
getCompoundContentRect wMetrics (visHScroll,visVScroll) itemRect
	| visHScroll && visVScroll = itemRect {rright=r',rbottom=b'}
	| visHScroll               = itemRect {          rbottom=b'}
	| visVScroll               = itemRect {rright=r'           }
	| otherwise                = itemRect
	where
		r'                 = rright  itemRect - osmVSliderWidth  wMetrics
		b'                 = rbottom itemRect - osmHSliderHeight wMetrics

getCompoundHScrollRect :: OSWindowMetrics -> (Bool,Bool) -> Rect -> Rect
getCompoundHScrollRect wMetrics (visHScroll,visVScroll) itemRect
	| not visHScroll = zero
	| visVScroll     = itemRect {rtop=b',rright=r'}
	| otherwise      = itemRect {rtop=b'}
	where
		r'       = rright  itemRect - osmVSliderWidth  wMetrics
		b'       = rbottom itemRect - osmHSliderHeight wMetrics

getCompoundVScrollRect :: OSWindowMetrics -> (Bool,Bool) -> Rect -> Rect
getCompoundVScrollRect wMetrics (visHScroll,visVScroll) itemRect
	| not visVScroll = zero
	| visHScroll     = itemRect {rleft=r',rbottom=b'}
	| otherwise      = itemRect {rleft=r'}
	where
		r'       = rright  itemRect - osmVSliderWidth  wMetrics
		b'       = rbottom itemRect - osmHSliderHeight wMetrics


getWindowContentRect :: OSWindowMetrics -> (Bool,Bool) -> Rect -> Rect
getWindowContentRect wMetrics (visHScroll,visVScroll) itemRect
	| visHScroll && visVScroll = itemRect {rright=r',rbottom=b'}
	| visHScroll               = itemRect {          rbottom=b'}
	| visVScroll               = itemRect {rright=r'           }
	| otherwise                = itemRect
	where
		r'                 = rright  itemRect - osmVSliderWidth  wMetrics
		b'                 = rbottom itemRect - osmHSliderHeight wMetrics

getWindowHScrollRect :: OSWindowMetrics -> (Bool,Bool) -> Rect -> Rect
getWindowHScrollRect wMetrics (visHScroll,visVScroll) (Rect {rleft=rleft,rtop=rtop,rright=rright,rbottom=rbottom})
	| not visHScroll = zero
	| otherwise      = Rect {rleft=rleft-1, rtop=b', rright=if visVScroll then r'+1 else rright+1, rbottom=rbottom+1}
	where
		r'       = rright  - osmVSliderWidth  wMetrics + 1
		b'       = rbottom - osmHSliderHeight wMetrics + 1

getWindowVScrollRect :: OSWindowMetrics -> (Bool,Bool) -> Rect -> Rect
getWindowVScrollRect wMetrics (visHScroll,visVScroll) (Rect {rleft=rleft,rtop=rtop,rright=rright,rbottom=rbottom})
	| not visVScroll = zero
	| otherwise      = Rect {rleft=r', rtop=rtop-1, rright=rright+1, rbottom=if visHScroll then b'+1 else rbottom+1}
	where
		r'       = rright  - osmVSliderWidth  wMetrics + 1
		b'       = rbottom - osmHSliderHeight wMetrics + 1


--	Access operations on WindowStateHandles:

getWindowStateHandleWIDS :: WindowStateHandle IF_MVAR(,ps) -> WIDS
#if MVAR
getWindowStateHandleWIDS wsH = wshIds wsH
#else
getWindowStateHandleWIDS wsH@(WindowStateHandle wshIds _) = wshIds
#endif

getWindowStateHandleWindowMode :: WindowStateHandle IF_MVAR(,ps) -> WindowMode
#if MVAR
getWindowStateHandleWindowMode wsH@(WindowStateHandle {wshHandle=Just (WindowHandle {whMode=whMode})}) = whMode
#else
getWindowStateHandleWindowMode wsH@(WindowStateHandle _ (Just (WindowLSHandle {wlsHandle=WindowHandle {whMode=whMode}}))) = whMode
#endif
	
getWindowStateHandleWindowKind :: WindowStateHandle IF_MVAR(,ps) -> WindowKind
#if MVAR
getWindowStateHandleWindowKind wsH@(WindowStateHandle {wshHandle=Just (WindowHandle {whKind=whKind})}) = whKind
#else
getWindowStateHandleWindowKind wsH@(WindowStateHandle _ (Just (WindowLSHandle {wlsHandle=WindowHandle {whKind=whKind}}))) = whKind
#endif

getWindowStateHandleWindowTitle :: WindowStateHandle IF_MVAR(,ps) -> Title
#if MVAR
getWindowStateHandleWindowTitle wsH@(WindowStateHandle {wshHandle=Just (WindowHandle {whTitle=whTitle})}) = whTitle
#else
getWindowStateHandleWindowTitle wsH@(WindowStateHandle _ (Just (WindowLSHandle {wlsHandle=WindowHandle {whTitle=whTitle}}))) = whTitle
#endif

getWindowStateHandleItemNrs :: WindowStateHandle IF_MVAR(,ps) -> [Int]
#if MVAR
getWindowStateHandleItemNrs wsH@(WindowStateHandle {wshHandle=Just (WindowHandle {whItemNrs=whItemNrs})}) = whItemNrs
#else
getWindowStateHandleItemNrs wsH@(WindowStateHandle _ (Just (WindowLSHandle {wlsHandle=WindowHandle {whItemNrs=whItemNrs}}))) = whItemNrs
#endif

getWindowStateHandleKeyFocus :: WindowStateHandle IF_MVAR(,ps) -> KeyFocus
#if MVAR
getWindowStateHandleKeyFocus wsH@(WindowStateHandle {wshHandle=Just (WindowHandle {whKeyFocus=whKeyFocus})}) = whKeyFocus
#else
getWindowStateHandleKeyFocus wsH@(WindowStateHandle _ (Just (WindowLSHandle {wlsHandle=WindowHandle {whKeyFocus=whKeyFocus}}))) = whKeyFocus
#endif

getWindowStateHandleWindowInfo :: WindowStateHandle ps -> WindowInfo
getWindowStateHandleWindowInfo wsH@(WindowStateHandle _ (Just (WindowLSHandle {wlsHandle=WindowHandle {whWindowInfo=whWindowInfo}}))) = whWindowInfo

getWindowStateHandleShow :: WindowStateHandle ps -> Bool
getWindowStateHandleShow wsH@(WindowStateHandle _ (Just (WindowLSHandle {wlsHandle=WindowHandle {whShow=whShow}}))) = whShow

getWindowStateHandleActive :: WindowStateHandle IF_MVAR(,ps) -> Bool
#if MVAR
getWindowStateHandleActive wsH@(WindowStateHandle {wshIds=WIDS {wActive=wActive}}) = wActive
#else
getWindowStateHandleActive wsH@(WindowStateHandle (WIDS {wActive=wActive}) _) = wActive
#endif

getWindowStateHandleDefaultId :: WindowStateHandle ps -> Maybe Id
getWindowStateHandleDefaultId wsH@(WindowStateHandle _ (Just (WindowLSHandle {wlsHandle=WindowHandle{whDefaultId=whDefaultId}})))
	= whDefaultId

getWindowStateHandleCancelId :: WindowStateHandle ps -> Maybe Id
getWindowStateHandleCancelId wsH@(WindowStateHandle _ (Just (WindowLSHandle {wlsHandle=WindowHandle{whCancelId=whCancelId}})))
	= whCancelId

getWindowStateHandleSelect :: WindowStateHandle IF_MVAR(,ps) -> Bool
#if MVAR
getWindowStateHandleSelect wsH@(WindowStateHandle {wshHandle=Just (WindowHandle {whSelect=whSelect})}) = whSelect
#else
getWindowStateHandleSelect wsH@(WindowStateHandle _ (Just (WindowLSHandle {wlsHandle=WindowHandle {whSelect=whSelect}}))) = whSelect
#endif

getWindowStateHandleSize :: WindowStateHandle IF_MVAR(,ps) -> Size
#if MVAR
getWindowStateHandleSize wsH@(WindowStateHandle {wshHandle=Just (WindowHandle {whSize=whSize})}) = whSize
#else
getWindowStateHandleSize wsH@(WindowStateHandle _ (Just (WindowLSHandle {wlsHandle=WindowHandle {whSize=whSize}}))) = whSize
#endif

getWindowStateHandleClosing :: WindowStateHandle IF_MVAR(,ps) -> Bool
#if MVAR
getWindowStateHandleClosing wsH@(WindowStateHandle {wshHandle=Just (WindowHandle {whClosing=whClosing})}) = whClosing
#else
getWindowStateHandleClosing wsH@(WindowStateHandle _ (Just (WindowLSHandle {wlsHandle=WindowHandle {whClosing=whClosing}}))) = whClosing
#endif

isWindowStateHandlePlaceHolder :: WindowStateHandle IF_MVAR(,ps) -> Bool
#if MVAR
isWindowStateHandlePlaceHolder wsH@(WindowStateHandle {wshHandle=Nothing}) = True
#else
isWindowStateHandlePlaceHolder wsH@(WindowStateHandle _ Nothing) = True
#endif
isWindowStateHandlePlaceHolder wsH = False

identifyWindowStateHandle :: WID -> WindowStateHandle IF_MVAR(,ps) -> Bool
identifyWindowStateHandle wid wsH = identifyWIDS wid (getWindowStateHandleWIDS wsH)

setWindowStateHandleWIDS :: WIDS -> WindowStateHandle IF_MVAR(,ps) -> WindowStateHandle IF_MVAR(,ps)
#if MVAR
setWindowStateHandleWIDS wids wsH = wsH {wshIds=wids}
#else
setWindowStateHandleWIDS wids (WindowStateHandle _ wlsH) = WindowStateHandle wids wlsH
#endif

setWindowStateHandleWindowTitle :: Title -> WindowStateHandle IF_MVAR(,ps) -> WindowStateHandle IF_MVAR(,ps)
#if MVAR
setWindowStateHandleWindowTitle title wsH@(WindowStateHandle {wshHandle=Just wH})
	= wsH {wshHandle=Just (wH {whTitle=title})}
#else
setWindowStateHandleWindowTitle title (WindowStateHandle wids (Just wlsH@(WindowLSHandle {wlsHandle=wH})))
	= WindowStateHandle wids (Just (wlsH {wlsHandle=wH {whTitle=title}}))
#endif

setWindowStateHandleItemNrs :: [Int] -> WindowStateHandle IF_MVAR(,ps) -> WindowStateHandle IF_MVAR(,ps)
#if MVAR
setWindowStateHandleItemNrs itemNrs wsH@(WindowStateHandle {wshHandle=Just wH})
	= wsH {wshHandle=Just (wH {whItemNrs=itemNrs})}
#else
setWindowStateHandleItemNrs itemNrs (WindowStateHandle wids (Just wlsH@(WindowLSHandle {wlsHandle=wH})))
	= WindowStateHandle wids (Just (wlsH {wlsHandle=wH {whItemNrs=itemNrs}}))
#endif

setWindowStateHandleActive :: Bool -> WindowStateHandle IF_MVAR(,ps) -> WindowStateHandle IF_MVAR(,ps)
#if MVAR
setWindowStateHandleActive active wsH@(WindowStateHandle {wshIds=wids})
	= wsH {wshIds=wids {wActive=active}}
#else
setWindowStateHandleActive active (WindowStateHandle wids wlsH)
	= WindowStateHandle (wids {wActive=active}) wlsH
#endif

setWindowStateHandleSelect :: Bool -> WindowStateHandle IF_MVAR(,ps) -> WindowStateHandle IF_MVAR(,ps)
#if MVAR
setWindowStateHandleSelect select wsH@(WindowStateHandle {wshHandle=Just wH})
	= wsH{wshHandle=Just (wH{whSelect=select})}
#else
setWindowStateHandleSelect select (WindowStateHandle wids (Just wlsH@(WindowLSHandle {wlsHandle=wH})))
	= WindowStateHandle wids (Just (wlsH {wlsHandle=wH {whSelect=select}}))
#endif
	
setWindowStateHandleSize :: Size -> WindowStateHandle IF_MVAR(,ps) -> WindowStateHandle IF_MVAR(,ps)
#if MVAR
setWindowStateHandleSize size wsH@(WindowStateHandle {wshHandle=Just wH})
	= wsH {wshHandle=Just (wH {whSize=size})}
#else
setWindowStateHandleSize size (WindowStateHandle wids (Just wlsH@(WindowLSHandle {wlsHandle=wH})))
	= WindowStateHandle wids (Just (wlsH {wlsHandle=wH {whSize=size}}))
#endif

setWindowStateHandleClosing :: Bool -> WindowStateHandle IF_MVAR(,ps) -> WindowStateHandle IF_MVAR(,ps)
#if MVAR
setWindowStateHandleClosing closing wsH@(WindowStateHandle {wshHandle=Just wH})
	= wsH {wshHandle=Just (wH {whClosing=closing})}
#else
setWindowStateHandleClosing closing (WindowStateHandle wids (Just wlsH@(WindowLSHandle {wlsHandle=wH})))
	= WindowStateHandle wids (Just (wlsH {wlsHandle=wH {whClosing=closing}}))
#endif


{-	Access operations on the margins and item space attributes of the window attributes.
	getWindow((H/V)Margin/ItemSpace)s type metrics atts
		retrieves the indicated attribute if present from the attribute list. If the attribute
		could not be found, the appropriate default value is returned. 
-}
getWindowHMargins :: WindowKind -> OSWindowMetrics -> [WindowAttribute IF_MVAR(,ls ps)] -> (Int,Int)
getWindowHMargins wKind wMetrics atts
	= (defaultLeft,defaultRight)
--	= getWindowHMarginAtt (snd (cselect isWindowHMargin (WindowHMargin defaultLeft defaultRight) atts))
	where
		(defaultLeft,defaultRight) = case wKind of
						IsDialog -> (osmHorMargin wMetrics,osmHorMargin wMetrics)
						other    -> (0,0)

getWindowVMargins :: WindowKind -> OSWindowMetrics -> [WindowAttribute IF_MVAR(,ls ps)] -> (Int,Int)
getWindowVMargins wKind wMetrics atts
	= (defaultTop,defaultBottom)
--	= getWindowVMarginAtt (snd (cselect isWindowVMargin (WindowVMargin defaultTop defaultBottom) atts))
	where
		(defaultTop,defaultBottom) = case wKind of
						IsDialog -> (osmVerMargin wMetrics,osmVerMargin wMetrics)
						other    -> (0,0)

getWindowItemSpaces :: WindowKind -> OSWindowMetrics -> [WindowAttribute IF_MVAR(,ls ps)] -> (Int,Int)
getWindowItemSpaces wKind wMetrics atts
	= (defaultHor,defaultVer)
--	= getWindowItemSpaceAtt (snd (cselect isWindowItemSpace (WindowItemSpace defaultHor defaultVer) atts))
	where
		(defaultHor,defaultVer) = case wKind of
						IsDialog -> (osmHorItemSpace wMetrics,osmVerItemSpace wMetrics)
						other    -> (0,0)


--	Search, get, and set WindowStateHandles.

getWindowHandlesActiveWindow :: WindowHandles IF_MVAR(,ps) -> Maybe WIDS
getWindowHandlesActiveWindow wHs@(WindowHandles {whsWindows=wsHs})
	= fmap getWindowStateHandleWIDS (find (wActive . getWindowStateHandleWIDS) wsHs)
		

--	getWindowHandlesActiveModalDialog assumes that all modal dialogues are at the front of the list
getWindowHandlesActiveModalDialog :: WindowHandles IF_MVAR(,ps) -> Maybe WIDS
getWindowHandlesActiveModalDialog wHs@(WindowHandles {whsWindows=[]})
	= Nothing
getWindowHandlesActiveModalDialog wHs@(WindowHandles {whsWindows=(wsH:wsHs)})	
	| mode /= Modal = Nothing
	| otherwise    	= Just wids
	where
		mode = getWindowStateHandleWindowMode wsH
		wids = getWindowStateHandleWIDS wsH

hasWindowHandlesWindow :: WID -> WindowHandles IF_MVAR(,ps) -> Bool
hasWindowHandlesWindow wid wHs@(WindowHandles {whsWindows=whsWindows})
	= haswindow wid whsWindows
	where		
		haswindow :: WID -> [WindowStateHandle IF_MVAR(,ps)] -> Bool
		haswindow wid (wsH:wsHs) = identifyWIDS wid wIds || haswindow wid wsHs
			where wIds = getWindowStateHandleWIDS wsH
		haswindow _ _ = False

getWindowHandlesWindow :: WID -> WindowHandles IF_MVAR(,ps) -> (Bool,WindowStateHandle IF_MVAR(,ps),WindowHandles IF_MVAR(,ps))
getWindowHandlesWindow wid wHs@(WindowHandles {whsWindows=whsWindows})
	= (ok,wsH,wHs {whsWindows=whsWindows1})
	where
		(ok,wsH,whsWindows1) = getwindow wid whsWindows
		
		getwindow :: WID -> [WindowStateHandle IF_MVAR(,ps)] -> (Bool,WindowStateHandle IF_MVAR(,ps),[WindowStateHandle IF_MVAR(,ps)])
		getwindow wid (wsH:wsHs)
			| identifyWIDS wid wIds
			      = (True,wsH,(WindowStateHandle wIds Nothing):wsHs)
			| otherwise
			      = let (found,wsH1,wsHs1) = getwindow wid wsHs
			        in  (found,wsH1,wsH:wsHs1)
			where
				wIds = getWindowStateHandleWIDS wsH
		getwindow _ _
			= (False,dummyWindowStateHandle,[])

removeWindowHandlesWindow :: WID -> WindowHandles IF_MVAR(,ps) -> (Bool,WindowStateHandle IF_MVAR(,ps),WindowHandles IF_MVAR(,ps))
removeWindowHandlesWindow wid wHs@(WindowHandles {whsWindows=whsWindows})
	= (ok,wsH,wHs {whsWindows=whsWindows1})
	where
		(ok,wsH,whsWindows1) = remove (identifyWindowStateHandle wid) dummyWindowStateHandle whsWindows
		
		identifyWindowStateHandle :: WID -> WindowStateHandle IF_MVAR(,ps) -> Bool
		identifyWindowStateHandle wid wsH = identifyWIDS wid wIds
			where wIds = getWindowStateHandleWIDS wsH

setWindowHandlesWindow :: WindowStateHandle IF_MVAR(,ps) -> WindowHandles IF_MVAR(,ps) -> WindowHandles IF_MVAR(,ps)
setWindowHandlesWindow wsH wHs@(WindowHandles {whsWindows=whsWindows})
	| isWindowStateHandlePlaceHolder wsH
		= windowaccessFatalError "setWindowHandlesWindow" "WindowStateHandle argument should not be a place holder"
	| otherwise
		= wHs{whsWindows=setwindow (getWindowStateHandleWIDS wsH) wsH whsWindows}
	where		
		setwindow :: WIDS -> WindowStateHandle IF_MVAR(,ps) -> [WindowStateHandle IF_MVAR(,ps)] -> [WindowStateHandle IF_MVAR(,ps)]
		setwindow wids' wsH' (wsH:wsHs)
			| wids/=wids'
				= wsH:setwindow wids' wsH' wsHs
			| isWindowStateHandlePlaceHolder wsH
				= wsH':wsHs
			| otherwise
				= windowaccessFatalError "setWindowHandlesWindow" "place holder expected instead of WindowStateHandle"
			where
				wids = getWindowStateHandleWIDS wsH
		setwindow _ _ _
			= windowaccessFatalError "setWindowHandlesWindow" "place holder not found"

addBehindWindowHandlesWindow :: WID -> WindowStateHandle IF_MVAR(,ps) -> WindowHandles IF_MVAR(,ps) -> (WIDS,WindowHandles IF_MVAR(,ps))
addBehindWindowHandlesWindow behindWID wsH wHs@(WindowHandles {whsWindows=whsWindows})
	| isWindowStateHandlePlaceHolder wsH
		= windowaccessFatalError "addBehindWindowHandlesWindow" "WindowStateHandle argument should not be a place holder"
	| otherwise
		= let (behindWIDS,whsWindows1) = stackBehind behindWID wsH whsWindows
	          in  (behindWIDS,wHs {whsWindows=whsWindows1})
	where		
		stackBehind :: WID -> WindowStateHandle IF_MVAR(,ps) -> [WindowStateHandle IF_MVAR(,ps)] -> (WIDS,[WindowStateHandle IF_MVAR(,ps)])
		stackBehind behindWID wsH (wsH':wsHs)
			| not (identifyWIDS behindWID wids')
				= let (behindWIDS,wsHs1) = stackBehind behindWID wsH wsHs
			          in  (behindWIDS,wsH':wsHs1)
			| otherwise
				= (wids',wsH':wsH:wsHs)
			where
				wids' = getWindowStateHandleWIDS wsH'
		stackBehind _ _ _
			= windowaccessFatalError "addBehindWindowHandlesWindow" "behind window could not be found"

addWindowHandlesWindow :: Index -> WindowStateHandle IF_MVAR(,ps) -> WindowHandles IF_MVAR(,ps) -> WindowHandles IF_MVAR(,ps)
addWindowHandlesWindow index wsH wHs@(WindowHandles {whsWindows=whsWindows})
	= wHs {whsWindows=insert (max 0 index) wsH whsWindows}
	where
		insert :: Index -> x -> [x] -> [x]
		insert 0 x ys
			= x:ys
		insert i x (y:ys)
			= y:insert (i-1) x ys
		insert _ x _
			= [x]

addWindowHandlesActiveWindow :: WindowStateHandle IF_MVAR(,ps) -> WindowHandles IF_MVAR(,ps) -> WindowHandles IF_MVAR(,ps)
addWindowHandlesActiveWindow wsH wHs@(WindowHandles {whsWindows=whsWindows})
	= wHs {whsWindows=wsH:whsWindows}


{-	Checking WindowBounds:
-}
checkZeroWindowHandlesBound :: WindowHandles IF_MVAR(,ps) -> Bool
checkZeroWindowHandlesBound (WindowHandles {whsNrWindowBound=whsNrWindowBound})
	= zeroBound whsNrWindowBound

decreaseWindowHandlesBound :: WindowHandles IF_MVAR(,ps) -> WindowHandles IF_MVAR(,ps)
decreaseWindowHandlesBound wHs@(WindowHandles {whsNrWindowBound=whsNrWindowBound})
	= wHs {whsNrWindowBound=decBound whsNrWindowBound}


{-	getInitActiveControl retrieves the OSWindowPtr of the control that has the initial input focus.
	It is assumed that the control identified by the WindowInitActive attribute exists.
-}
getInitActiveControl :: WindowHandle IF_MVAR(,ls ps) -> (OSWindowPtr,WindowHandle IF_MVAR(,ls ps))
getInitActiveControl wH@(WindowHandle {whItems=itemHs,whAtts=whAtts})
	= (if isJust maybeItemPtr then fromJust maybeItemPtr else osNoWindowPtr,wH {whItems=itemHs1})
	where
	{-	(hasInitActiveAtt,initActiveAtt)= cselect isWindowInitActive undefined whAtts
		initActiveId			= if hasInitActiveAtt (Just (getWindowInitActiveAtt initActiveAtt)) Nothing
	REPLACED BY:
	-}	initActiveId			= Nothing
		(maybeItemPtr,itemHs1)		= accessWElementHandles (condition initActiveId) (toSt wItemPtr) itemHs
		
		condition initActiveId@(Just _) itemH
			= initActiveId==wItemId itemH
		condition _ itemH
			= wItemKind itemH==IsEditControl


{-	Determine the list of window items that can obtain the keyboard input focus. -}

getWElementKeyFocusIds :: Bool -> [WElementHandle IF_MVAR(,ls ps)] -> [FocusItem]
getWElementKeyFocusIds shownContext (itemH:itemHs) =
	(getWElementKeyFocusIds' shownContext itemH)   ++
	(getWElementKeyFocusIds  shownContext itemHs)
	where
#if MVAR
		getWElementKeyFocusIds' :: Bool -> WElementHandle -> [FocusItem]
#else
		getWElementKeyFocusIds' :: Bool -> WElementHandle ls ps -> [FocusItem]
#endif
		getWElementKeyFocusIds' shownContext itemH@(WItemHandle {wItemNr=wItemNr,wItemKind=wItemKind,wItemShow=wItemShow,wItemAtts=wItemAtts,wItems=wItems})
			| wItemKind==IsEditControl   	= focus
			| keySensitive && hasKeyAtt  	= focus
			| otherwise 			= getWElementKeyFocusIds (shownContext && wItemShow) wItems			
			where
				focus		= [FocusItem {focusNr=wItemNr,focusShow=shownContext}]
				hasKeyAtt	= any isControlKeyboard wItemAtts
				keySensitive	= wItemKind==IsCustomControl

		getWElementKeyFocusIds' shownContext (WListLSHandle itemHs) =
			getWElementKeyFocusIds shownContext itemHs
#if MVAR
#else
		getWElementKeyFocusIds' shownContext (WExtendLSHandle ls1 itemHs) =
			getWElementKeyFocusIds shownContext itemHs			

		getWElementKeyFocusIds' shownContext (WChangeLSHandle ls1 itemHs) =
			getWElementKeyFocusIds shownContext itemHs			
#endif

getWElementKeyFocusIds _ _ = []


{-	Generate internal numbers for all WElementHandles which wItemNr==0. -}

genWElementItemNrs :: [Int] -> [WElementHandle IF_MVAR(,ls ps)] -> ([Int],[WElementHandle IF_MVAR(,ls ps)])
genWElementItemNrs nrs (itemH:itemHs)
	= (nrs2,itemH1:itemHs1)
	where
		(nrs1,itemH1)  = genWElementNrs     nrs  itemH
		(nrs2,itemHs1) = genWElementItemNrs nrs1 itemHs
		
		genWElementNrs :: [Int] -> WElementHandle IF_MVAR(,ls ps) -> ([Int],WElementHandle IF_MVAR(,ls ps))
		genWElementNrs nrs wItemH@(WItemHandle {wItemNr=wItemNr,wItems=wItems})
			= let (nrs1,itemHs)	= genWElementItemNrs nrs wItems
			  in if wItemNr/=0 then (nrs1,wItemH{wItems=itemHs})
			     else (tail nrs1,wItemH{wItemNr=head nrs1,wItems=itemHs})
		
		genWElementNrs nrs (WListLSHandle itemHs)
			= let (nrs1,itemHs1)  = genWElementItemNrs nrs itemHs
			  in  (nrs1,WListLSHandle itemHs1)
		
#if MVAR
#else
		genWElementNrs nrs (WExtendLSHandle addLS itemHs)
			= let (nrs1,itemHs1) = genWElementItemNrs nrs itemHs
			  in  (nrs1,WExtendLSHandle addLS itemHs1)
		
		genWElementNrs nrs (WChangeLSHandle newLS itemHs)
			= let (nrs1,itemHs1) = genWElementItemNrs nrs itemHs
			  in  (nrs1,WChangeLSHandle newLS itemHs1)
#endif

genWElementItemNrs nrs _
	= (nrs,[])



{-	General access functions on WItemHandles in (WindowStateHandle IF_MVAR(,ps)) and [WElementHandle IF_MVAR(,ls ps)] respectively. 
	The (St   (WElementHandle IF_MVAR(,ls ps)) x) is applied to a WItemHandle iff
	the (Cond (WElementHandle IF_MVAR(,ls ps))) holds.
	Here we take advantage of the fact that GHC supports Rank2 polymorphism.
	accessWItemHandle     aborts if applied to a window placeholder.
	accessWElementHandles returns Nothing if no WItemHandle could be found.
-}
#if MVAR
accessWItemHandle :: String -> Cond WElementHandle -> St WElementHandle x -> WindowStateHandle -> (Maybe x,WindowStateHandle)
accessWItemHandle _ cond f wsH@(WindowStateHandle {wshHandle=Just wH})
#else
accessWItemHandle :: String
                  -> (forall ls. Cond (WElementHandle ls ps))
                  -> (forall ls. St   (WElementHandle ls ps) x)
                  -> WindowStateHandle ps
                  -> (Maybe x,WindowStateHandle ps)
accessWItemHandle _ cond f (WindowStateHandle wids (Just wlsH@(WindowLSHandle {wlsHandle=wH})))
#endif
	= let (maybe,itemHs) = accessWElementHandles cond f (whItems wH)
#if MVAR
	  in  (maybe,wsH {wshHandle=Just (wH {whItems=itemHs})})
#else
	  in  (maybe,WindowStateHandle wids (Just (wlsH {wlsHandle=wH {whItems=itemHs}})))
#endif
	where
#if MVAR
		accessWElementHandles :: Cond WElementHandle -> St WElementHandle x -> [WElementHandle] -> (Maybe x,[WElementHandle])
#else
		accessWElementHandles :: (forall ls. Cond (WElementHandle ls ps)) -> (forall ls. St (WElementHandle ls ps) x) -> [WElementHandle ls ps]
		                      -> (Maybe x,[WElementHandle ls ps])
#endif
		accessWElementHandles cond f (itemH:itemHs)
			= let (maybe,itemH1) = accessWElementHandle cond f itemH
			  in  if   isJust maybe
			      then (maybe,itemH1:itemHs)
			      else let (maybe1,itemHs1) = accessWElementHandles cond f itemHs
			           in  (maybe1,itemH1:itemHs1)
			where
#if MVAR
				accessWElementHandle :: Cond WElementHandle -> St WElementHandle x -> WElementHandle -> (Maybe x,WElementHandle)
#else
				accessWElementHandle :: (forall ls. Cond (WElementHandle ls ps)) -> (forall ls. St (WElementHandle ls ps) x) -> WElementHandle ls ps
				                     -> (Maybe x,WElementHandle ls ps)
#endif
				accessWElementHandle cond f (WListLSHandle itemHs)
					= let (maybe,itemHs1) = accessWElementHandles cond f itemHs
					  in  (maybe,WListLSHandle itemHs1)
#if MVAR
#else
				accessWElementHandle cond f (WExtendLSHandle addLS itemHs)
					= let (maybe,itemHs1) = accessWElementHandles cond f itemHs
					  in  (maybe,WExtendLSHandle addLS itemHs1)
				
				accessWElementHandle cond f (WChangeLSHandle newLS itemHs)
					= let (maybe,itemHs1) = accessWElementHandles cond f itemHs
					  in  (maybe,WChangeLSHandle newLS itemHs1)
#endif
				accessWElementHandle cond f itemH
					| cond itemH
						= let (x,itemH1) = f itemH
						  in  (Just x,itemH1)
					| otherwise
						= (Nothing,itemH)
		accessWElementHandles _ _ []
			= (Nothing,[])

accessWItemHandle functionName _ _ _
	= windowaccessFatalError functionName "window placeholder not expected"

#if MVAR
accessWElementHandles :: Cond WElementHandle -> St WElementHandle x -> [WElementHandle] -> (Maybe x,[WElementHandle])
#else
accessWElementHandles :: (forall ls. Cond (WElementHandle ls ps))
                      -> (forall ls. St   (WElementHandle ls ps) x)
                      -> [WElementHandle ls ps]
                      -> (Maybe x,[WElementHandle ls ps])
#endif
accessWElementHandles cond f (itemH:itemHs)
	= let (maybe,itemH1) = accessWElementHandle cond f itemH
	  in  if   isJust maybe
	      then (maybe,itemH1:itemHs)
	      else let (maybe1,itemHs1) = accessWElementHandles cond f itemHs
	           in  (maybe1,itemH1:itemHs1)
	where
#if MVAR
		accessWElementHandle :: Cond WElementHandle -> St WElementHandle x -> WElementHandle -> (Maybe x,WElementHandle)
#else
		accessWElementHandle :: (forall ls. Cond (WElementHandle ls ps)) -> (forall ls. St (WElementHandle ls ps) x) -> WElementHandle ls ps
		                     -> (Maybe x,WElementHandle ls ps)
#endif
		accessWElementHandle cond f (WListLSHandle itemHs)
			= let (maybe,itemHs1) = accessWElementHandles cond f itemHs
			  in  (maybe,WListLSHandle itemHs1)
#if MVAR
#else
		accessWElementHandle cond f (WExtendLSHandle addLS itemHs)
			= let (maybe,itemHs1) = accessWElementHandles cond f itemHs
			  in  (maybe,WExtendLSHandle addLS itemHs1)
		
		accessWElementHandle cond f (WChangeLSHandle newLS itemHs)
			= let (maybe,itemHs1) = accessWElementHandles cond f itemHs
			  in  (maybe,WChangeLSHandle newLS itemHs1)
#endif
		accessWElementHandle cond f itemH
			| cond itemH
				= let (x,itemH1) = f itemH
				  in  (Just x,itemH1)
			| otherwise
				= (Nothing,itemH)
accessWElementHandles _ _ []
	= (Nothing,[])
	
	
getFinalModalLS :: WID -> FinalModalLS -> Maybe ls
getFinalModalLS wid (FinalModalLS wids ls)
	| identifyWIDS wid wids = Just (unsafeCoerce# ls)
	| otherwise		= Nothing
